from Products.PTKBase.PortalCatalog import PortalCatalog
from Products.PTKBase.PortalFolder import *


default_member_content = """\
Default page for %s

  This is the default document created for you when 
  you joined this community.

  To change the content just select "Manage Document"
  in the Tool Box on the left.
"""

class DemoPortalBase:
    ' '

    __ac_permissions=(
        ('Manage portal',
         ('install', 'portal_reconfig', 'refresh_interface', 'apply_wizard'),
         ('Owner',),
         )
        )
        
    def install(self, REQUEST=None):
        """Set up initial components  --  called by ZClass constructor"""

        if REQUEST is None:
            REQUEST = self.REQUEST

        # Add Members folder
        self.manage_addProduct['PTKBase'].manage_addPortalFolder(
            self, 'Members')

        # Determine membership storage and create acl_users
        #  Because SQLMemberFolder requires nonstandard arguments, I cannot
        #  use PTKBase.PortalBase.addMemberFolder
        db = REQUEST['database']
        if db == 'ZODB':
            self.MembersClass = self.ZODBMemberFolder
            acl_users = self.MembersClass()
        else:
            self.MembersClass = self.SQLMemberFolder
            acl_users = self.MembersClass(db)
        # I'm not sure why this is done via this() nor what __allow_groups__
        # is for, but this is what addMemberFolder does, so I do as well.
        self.this()._setObject('acl_users', acl_users)
        self.this().__allow_groups__=self.acl_users
        
        # Install PortalCatalog
        id = 'SiteIndex'
        catalog = PortalCatalog(id, '')
        self._setObject(id, catalog)
        catalog = self.SiteIndex
        catalog.initialize()
        
        # Create roles
        self._addRole('Member')
        self._addRole('Reviewer')
        self._addRole('Contributer')

        # Give permissions out
        self.manage_role('Owner', [
            'View management screens',
            'View private portal content',
            'Add portal content',
            'Add Folders',
            'Delete objects',
            'FTP access',
            'Manage properties',
            'Undo changes',
            'Request review'
            ])
        self.manage_role('Reviewer', [
            'Review portal content',
            'View pending portal content',
            ])
        self.manage_role('Contributer', [
            'Contribute portal content',
            ])
        self.manage_role('Anonymous', [
            'View published portal content',
            ])
        self.manage_role('Member', [
            'View published portal content',
            ])


    def portal_reconfig(self, REQUEST):
        "Accept changes from portal_config"
        self.propertysheets.PortalProperties.manage_changeProperties(REQUEST)
        self.MailHost.smtpHost = REQUEST['smtp_server']
        return self.portal_config(self, REQUEST,
                                  portal_status_message="Settings changed")
        
    def before_review(self, changed_object, action, review_state, comments,
                      REQUEST):
        # Someone has just tried to change the reviewed_state of the object.
        # This is an opportunity to modify the change before it takes place,
        # or reject it entierly.
        #
        # 'changed_object' is a slight misnomner-- it hasn't changed _yet_.
        if REQUEST.AUTHENTICATED_USER.has_permission(
            'Contribute portal content', changed_object) and review_state=='pending':
            # This user is a contributer-- skip reviewing
            review_state = 'published'
            action = 'Item published'
        return action, review_state, comments
    
    def personalize(self, REQUEST):
        "Called by personalize_form to commit changes"
        user = REQUEST.AUTHENTICATED_USER
        user.setMemberProperties(REQUEST)

        return self.personalize_form(self, REQUEST,
                                     portal_status_message="Member changed.")

    def getRoster(self, REQUEST):
        """
        Return a list of the usernames of those users who have made themselves
        "listed".  If Manager, return a list of all usernames.
        """
        # Consider changing this to check for a permission rather than a role
        if REQUEST.AUTHENTICATED_USER.has_role('Manager'):
            return self.acl_users.getUserNames()
        names = []
        for user in self.acl_users.getUsers():
            if user.listed:
                names.append(user.getUserName())
        return names


    # Portal Wizard methods

    def refresh_interface(self, interface):
        """
        Replace the present interface with the one specified.
        This can be used to update an unchanged interface when the ZClasses
        have been upgraded.
        """

        # Copy from the specified Interface Folder to the Portal instance.
        # This will OVERWRITE existing objects.
        source = self.Interfaces[interface]
        for ob in source.objectValues():
            ob = ob._getCopy(source)
            try:   id = ob.id()
            except:id = ob.id
            if hasattr(self.aq_base, id):
                self._delObject(id)
            self._setObject(id, ob)

        # Wizards need to be handled way better XXX
        self.Wizards.manage_permission('View',['Member'],1) # let Members see Wizards

    def register_hook(self, REQUEST):
        """
        Called when a new user is registered
        """
        name=REQUEST['username']
        
        # Create Member's home folder
        self.manage_addProduct['PTKBase'].manage_addPortalFolder(
            self.Members, name)
        f=getattr(self.Members,name)

        # set permissions on home folder
        f.manage_permission('View', ['Owner','Manager','Reviewer'])
        
	# Create Member's home page
        f.manage_addProduct['PTKBase'].addDocument(
            f,
            'index_html',
            name+"'s Home",
            name+"'s front page",
            "structured-text",
            default_member_content % (name)
            )

        # Grant ownership to Member
        f.manage_setLocalRoles(name, ['Owner'])
        
    def apply_wizard(self, REQUEST):
        "Apply changes from filling in Portal Wizard"

        if not REQUEST.AUTHENTICATED_USER.has_role('Manager', self):
            raise 'Unauthorized'
        
        interface = REQUEST['interface']
        username = getattr(REQUEST, 'username', None)
        password = getattr(REQUEST, 'password', None)
        confirm = getattr(REQUEST, 'confirm', None)
	smtp_server = REQUEST['smtp_server']
        
        # Save info from Request in properties.
        self.propertysheets.PortalProperties.manage_changeProperties(REQUEST)

	# Create MailHost for mailing items from the portal
	self.manage_addMailHost('MailHost', smtp_host=smtp_server)

        if username:
            # Create manager's account.
            if password != confirm:
                raise "Password mismatch"
            self.acl_users.addMember(
                username, password, ['Member', 'Manager'], [])

            # Create manager's home folder
            self.manage_addProduct['PTKBase'].manage_addPortalFolder(
                self.Members, username)
            f=getattr(self.Members, username)

	# Create manager's default page
	# f.manage_addDTMLMethod('index_html', title=username, file=self.memberdefault_html)

        # Copy the interface
        self.refresh_interface(interface)
